#include "gamemanager.h"

GameManager::GameManager()
{
}
bool GameManager::validBinaryPath(){
    bool validPath = false;
    if(Settings::warzonePath.length() > 0 && QFile::exists(Settings::warzonePath)){
        validPath = true;
    }
    return validPath;
}
QString GameManager::findWarzone(){

    QString warzonePath = "";
#ifdef Q_OS_LINUX
    warzonePath = "/usr/games/warzone2100";
#endif

#ifdef Q_OS_MAC
    warzonePath = "/Applications/Warzone.app";
#endif

#ifdef Q_OS_WIN
    warzonePath = "C:\\Program Files\\Warzone 2100\\warzone2100.exe";
#endif
    return warzonePath;
}

QVector<class Mod> GameManager::getAvailableMods(){
    return this->availableMods;
}

QString GameManager::findModPath(){
    QString modPath = "";

#ifdef Q_OS_LINUX
    modPath = QString(getenv("HOME")) + "/.warzone2100-2.2/mods";
#endif

#ifdef Q_OS_MAC
    modPath = QString(getenv("HOME")) + "/Library/Application Support/Warzone 2100 2.2/mods";
#endif

#ifdef Q_OS_WIN
    modPath = "C:/Program Files/Warzone 2100/mods";
#endif

    QDir dir(modPath);
    if(!dir.exists()) {
        modPath = "";
    }
    return modPath;
}

QString GameManager::findDataPath(){
    QString dataPath = "";

#ifdef Q_OS_LINUX
    dataPath = QString(getenv("HOME")) + "/.warzone2100-2.2/";
#endif

#ifdef Q_OS_MAC
    dataPath = QString(getenv("HOME")) + "/Library/Application Support/Warzone 2100 2.2/";
#endif

#ifdef Q_OS_WIN
    dataPath = QString(getenv("USERPROFILE")) + "/My Documents/Warzone 2100 2.3";
#endif

    QDir dir(dataPath);
    if(!dir.exists()) {
        dataPath = "";
    }
    return dataPath;
}

void GameManager::loadAvailableMods(){

    QStringList dirs;
    dirs << "" << "/autoload" << "/global" << "/multiplay" << "/campaign" << "/global/autoload" << "/multiplay/autoload" << "//campaign/autoload";
    QStringList filter;
    filter << "*.wz";

    for(int dirIndex = 0;dirIndex < dirs.length();dirIndex++){
        QDir dir = QDir(Settings::modPath + dirs.at(dirIndex));

        QStringList modList = dir.entryList(filter,QDir::Files,QDir::NoSort);

        for(int index = 0;index < modList.size();index++) {
            bool validMod = true;
            QString modFilename = modList.at(index);

            Mod mod;
            mod.modFileName = dir.absolutePath() + "/" + modFilename;
            mod.modName = modFilename.right(modFilename.lastIndexOf("/"));
            if(mod.modName.contains(".gmod.wz") || mod.modName.contains(".mmod.wz") || mod.modName.contains(".cmod.wz")){
                mod.modName = mod.modName.left(mod.modName.lastIndexOf("."));
                mod.modName = mod.modName.left(mod.modName.lastIndexOf("."));
            }
            else{
                mod.modName = mod.modName.left(mod.modName.lastIndexOf("."));
            }
            if(((QString)dirs.at(dirIndex)).indexOf("autoload") > -1){
                mod.autoload = true;
            }
            else{
                mod.autoload = false;
            }


            if(modFilename.contains(".gmod.wz",Qt::CaseInsensitive) || ((QString)dirs.at(dirIndex)).indexOf("global") > -1) {
                mod.modType = Mod::Global;
            }
            else if(modFilename.contains(".cmod.wz",Qt::CaseInsensitive) || ((QString)dirs.at(dirIndex)).indexOf("campaign") > -1){
                mod.modType = Mod::Campaign;
            }
            else if(modFilename.contains(".mmod.wz",Qt::CaseInsensitive) || ((QString)dirs.at(dirIndex)).indexOf("multiplay") > -1){
                mod.modType = Mod::Multiplay;
            }
            else{
                validMod = false;
            }

            if(validMod){
                availableMods.append(mod);
            }
        }
    }
}

void GameManager::startGame(ModProfile p){
    bool success = true;
    QStringList options;
    for(int index = 0;index < p.orderedMods.size();index++) {
        Mod mod = p.orderedMods.at(index);
        switch(mod.modType){
        case Mod::Global:
            options << "--mod" << mod.modFileName.mid(mod.modFileName.lastIndexOf("/") + 1,mod.modFileName.length() - mod.modFileName.lastIndexOf("/"));
            break;
        case Mod::Multiplay:
            options << "--mod_mp" << mod.modFileName.mid(mod.modFileName.lastIndexOf("/") + 1,mod.modFileName.length() - mod.modFileName.lastIndexOf("/"));;
            break;
        case Mod::Campaign:
            options << "--mod_ca" << mod.modFileName.mid(mod.modFileName.lastIndexOf("/") + 1,mod.modFileName.length() - mod.modFileName.lastIndexOf("/"));;
            break;
        }
    }

    if(!Settings::debugAll){
        if(Settings::debug3D){
            options << "--debug=3d";
        }
        if(Settings::debugError){
            options << "--debug=error";
        }
        if(Settings::debugMain){
            options << "--debug=main";
        }
        if(Settings::debugMemory){
            options << "--debug=memory";
        }
        if(Settings::debugNet){
            options << "--debug=net";
        }
        if(Settings::debugScript){
            options << "--debug=script";
        }
        if(Settings::debugSound){
            options << "--debug=sound";
        }
        if(Settings::debugTexture){
            options << "--debug=texture";
        }
        if(Settings::debugVideo){
            options << "--debug=video";
        }
        if(Settings::debugWZ){
            options << "--debug=wz";
        }
    }
    else{
        options << "--debug=all";
    }
    if(Settings::cheatsEnabled)
    {
        options << "--cheat";
    }
    if(Settings::configDirEnabled){
        options << "--configdir" << Settings::configDir;
    }
    if(Settings::dataDirEnabled){
        options << "--datadir" << Settings::dataDir;
    }
    if(Settings::crash){
        options << "--crash";
    }
    if(Settings::fullscreen){
        options << "--fullscreen";
    }
    else{
        options << "--window";
    }
    if(Settings::gameTypeEnabled){
        if(Settings::gameType == "Alpha Campaign"){
            options << "--game=CAM_1A";
        }
        else if(Settings::gameType == "Beta Campaign"){
            options << "--game=CAM_2A";
        }
        else if(Settings::gameType == "Gamma Campaign"){
            options << "--game=CAM_3A";
        }
        else if(Settings::gameType == "Tutorial"){
            options << "--game=TUTORIAL3";
        }
        else if(Settings::gameType == "Fast Play"){
            options << "--game=FASTPLAY";
        }
    }
    if(Settings::joinMPEnabled){
        QString mphost = "--join=" + Settings::hostName;
        options << mphost;
    }
    if(Settings::logFileEnabled){
        QString debugFile = "--debugfile=" + Settings::logFile;
        options << debugFile;
    }
    if(Settings::resolutionEnabled){
        QString width;
        width.setNum(Settings::width);
        QString height;
        height.setNum(Settings::height);

        QString res = "--resolution=" + width + "x" + height;
        options << res;
    }
    if(Settings::savedGameEnabled){
        QString saveFile = "--savegame=" + Settings::savedGame + ".gam";
        options << saveFile;
    }
    if(Settings::selfTest){
        options << "--selftest";
    }
    if(Settings::shadows){
        options << "--shadows";
    }
    else{
        options << "--noshadows";
    }
    if(Settings::sound){
        options << "--sound";
    }
    else{
        options << "--nosound";
    }
    if(Settings::disableAsserts){
        options << "--noassert";
    }
    if(Settings::hostScreen){
        options << "--host";
    }
    if(Settings::flushDebug){
        options << "--flush-debug-stderr";
    }
    QString command  = Settings::warzonePath;

#ifdef Q_OS_MAC
    command += "/Contents/MacOS/Warzone";
#endif

    QProcess *process = new QProcess(0);

    QString workDir = command.left(command.lastIndexOf("/"));

    process->setWorkingDirectory(workDir);
    if(!process->startDetached(command,options)) {
        success = false;
    }
    delete process;
}

