#include "installmodwindow.h"
#include "ui_installmodwindow.h"

InstallModWindow::InstallModWindow(QWidget *parent) :
    QDialog(parent),
    m_ui(new Ui::InstallModWindow)
{
    m_ui->setupUi(this);
}

InstallModWindow::~InstallModWindow()
{
    delete m_ui;
}

void InstallModWindow::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
void InstallModWindow::on_browseButton_clicked() {
    QFileDialog *dia = new QFileDialog(this,Qt::Dialog);
    QStringList filters;
    filters << "Warzone 2100 Mods (*.wz)" << "Any files (*)";
    dia->setNameFilters(filters);
    switch(dia->exec()) {
        case QFileDialog::Accepted:
            this->m_ui->modLocation->setText(dia->selectedFiles().first());
            if(this->m_ui->modLocation->text().indexOf(".gmod.") != -1){
                this->m_ui->globalRadio->setChecked(true);
                this->m_ui->globalRadio->setEnabled(false);
                this->m_ui->campRadio->setEnabled(false);
                this->m_ui->multiplayRadio->setEnabled(false);
            }
            else if(this->m_ui->modLocation->text().indexOf(".cmod.") != -1){
                this->m_ui->campRadio->setChecked(true);
                this->m_ui->globalRadio->setEnabled(false);
                this->m_ui->campRadio->setEnabled(false);
                this->m_ui->multiplayRadio->setEnabled(false);
            }
            else if(this->m_ui->modLocation->text().indexOf(".mmod.") != -1){
                this->m_ui->multiplayRadio->setChecked(true);
                this->m_ui->globalRadio->setEnabled(false);
                this->m_ui->campRadio->setEnabled(false);
                this->m_ui->multiplayRadio->setEnabled(false);
            }
            else{
                this->m_ui->globalRadio->setEnabled(true);
                this->m_ui->campRadio->setEnabled(true);
                this->m_ui->multiplayRadio->setEnabled(true);
            }
            break;
    }
}

void InstallModWindow::on_installButton_clicked() {
    if(this->m_ui->modLocation->text().trimmed().length() > 0) {

        if(this->m_ui->globalRadio->isChecked() || this->m_ui->campRadio->isChecked() || this->m_ui->multiplayRadio->isChecked()){
            if(this->m_ui->modLocation->text().indexOf("/") == -1) {
                QMessageBox::information(this,"Install Mod","The provided mod path is not valid.",QMessageBox::Ok,QMessageBox::NoButton);
            }
            else {
                QFile file(this->m_ui->modLocation->text());

                QString fileName = this->m_ui->modLocation->text().mid(this->m_ui->modLocation->text().lastIndexOf("/") + 1,-1);
                if(file.exists()) {
                    QString newFilename;
                    QString dirPath;
                    if(this->m_ui->globalRadio->isChecked()) {
                        newFilename = Settings::modPath + "/global/" + fileName;

                        if(m_ui->autoLoadCheck->isChecked()){
                            newFilename = Settings::modPath + "/global/autoload/" + fileName;
                            dirPath = Settings::modPath + "/global/autoload";
                        }
                        else{
                            newFilename = Settings::modPath + "/global/" + fileName;
                            dirPath = Settings::modPath + "/global";
                        }
                        QDir dir(dirPath);
                        if(!dir.exists()){
                            dir.mkpath(dirPath);
                        }
                    }
                    if(this->m_ui->campRadio->isChecked()) {
                        newFilename = Settings::modPath + "/campaign/" + fileName;

                        if(m_ui->autoLoadCheck->isChecked()){
                            newFilename = Settings::modPath + "/campaign/autoload/" + fileName;
                            dirPath = Settings::modPath + "/campaign/autoload";
                        }
                        else{
                            newFilename = Settings::modPath + "/campaign/" + fileName;
                            dirPath = Settings::modPath + "/campaign";
                        }
                        QDir dir(dirPath);
                        if(!dir.exists()){
                            dir.mkpath(dirPath);
                        }
                    }
                    if(this->m_ui->multiplayRadio->isChecked()) {
                        newFilename = Settings::modPath + "/multiplay/" + fileName;

                        if(m_ui->autoLoadCheck->isChecked()){
                            newFilename = Settings::modPath + "/multiplay/autoload/" + fileName;
                            dirPath = Settings::modPath + "/multiplay/autoload";
                        }
                        else{
                            newFilename = Settings::modPath + "/multiplay/" + fileName;
                            dirPath = Settings::modPath + "/multiplay";
                        }
                        QDir dir(dirPath);
                        if(!dir.exists()){
                            dir.mkpath(dirPath);
                        }
                    }
                    if(file.copy(file.fileName(),newFilename)) {
                        QMessageBox::information(this,"Install Mod","The mod has been successfully installed.",QMessageBox::Ok,QMessageBox::NoButton);
                        this->accept();
                    }
                    else {
                        QMessageBox::warning(this,"Install Mod","Error installing mod:\n" + file.errorString(),QMessageBox::Ok,QMessageBox::NoButton);
                    }
                }
            }
        }
        else { //mod type not selected
            QMessageBox::information(this,"Install Mod","Please select a mod type.",QMessageBox::Ok,QMessageBox::NoButton);
        }

    }
    else { //no mod selected
        QMessageBox::information(this,"Install Mod","Please select a mod.",QMessageBox::Ok,QMessageBox::NoButton);
    }
}

void InstallModWindow::on_cancelButton_clicked() {
    this->reject();
}
