#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    init();
}

MainWindow::~MainWindow()
{
    delete ui;
}
void MainWindow::init(){

#ifdef Q_OS_LINUX
    QString settingsDir = QString(getenv("HOME")) + "/.wz2100launcher";
    QDir dir(settingsDir);
    if(!dir.exists()) {
        dir.mkpath(settingsDir);
    }
#endif
    Settings *settings = new Settings();
    GameManager *gameManager = new GameManager();
    if(!settings->loadSettings()) {
        Settings::warzonePath = gameManager->findWarzone();
        Settings::modPath = gameManager->findModPath();
        Settings::dataPath = gameManager->findDataPath();
    }
    if(gameManager->validBinaryPath()){
        gameManager->loadAvailableMods();
    }
    else{
        Settings::warzonePath = gameManager->findWarzone();
        Settings::modPath = gameManager->findModPath();
        Settings::dataPath = gameManager->findDataPath();
        showSettings();
    }
    delete gameManager;
    delete settings;
}
void MainWindow::showSettings(){
    SettingsDialog *settingsDialog = new SettingsDialog(this);
    settingsDialog->exec();
    delete settingsDialog;
}
void MainWindow::on_startGameButton_clicked()
{
    Settings *settings = new Settings();
    GameManager *gm = new GameManager();

    settings->saveSettings();
    gm->startGame(settings->getProfile(Settings::selectedProfile));
    delete settings;
    delete gm;
}

void MainWindow::on_settingsButton_clicked()
{
    showSettings();
}

void MainWindow::on_modButton_clicked()
{
    ModProfileWindow *modProfileWindow = new ModProfileWindow(this);
    if(modProfileWindow->exec() == ModProfileWindow::Accepted){
        Settings::selectedProfile = modProfileWindow->getSelectedProfile();
    }
    delete modProfileWindow;
}

void MainWindow::on_websiteButton_clicked()
{
    QDesktopServices::openUrl(QUrl("http://www.wz2100.net"));
}

void MainWindow::on_exitButton_clicked()
{
    this->close();
}

void MainWindow::on_pushButton_clicked()
{
    CreditsDialog *creditsDialog = new CreditsDialog(this);
    creditsDialog->exec();
    delete creditsDialog;
}
