#include "modmanager.h"
#include "ui_modmanager.h"

ModManager::ModManager(QWidget *parent,bool newProfile, ModProfile profile) :
    QDialog(parent),
    m_ui(new Ui::ModManager)
{
    m_ui->setupUi(this);
    this->currentProfile = profile;
    init(newProfile);
}

ModManager::~ModManager()
{
    delete m_ui;
}

void ModManager::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void ModManager::init(bool newProfile){
    this->newProfile = newProfile;
    if(!newProfile){
        this->m_ui->profileName->setText(this->currentProfile.getProfileName());
    }
    GameManager *gameManager = new GameManager();
    gameManager->loadAvailableMods();
    QVector<class Mod> avaialableMods = gameManager->getAvailableMods();
    m_ui->campaignModList->setColumnCount(2);
    m_ui->campaignModList->hideColumn(1);
    m_ui->campaignModList->setColumnWidth(0,235);

    m_ui->multiplayModList->setColumnCount(2);
    m_ui->multiplayModList->hideColumn(1);
    m_ui->multiplayModList->setColumnWidth(0,235);

    m_ui->globalModList->setColumnCount(2);
    m_ui->globalModList->hideColumn(1);
    m_ui->globalModList->setColumnWidth(0,235);

    for(int index = 0;index < avaialableMods.size();index++){
        Mod mod = avaialableMods.at(index);
        QTableWidgetItem *nameItem;
        QTableWidgetItem *fileItem;
        switch(mod.modType){
            case Mod::Campaign:
                m_ui->campaignModList->insertRow(m_ui->campaignModList->rowCount());
                nameItem = new QTableWidgetItem();
                if(mod.autoload){
                    nameItem->setFlags(Qt::ItemIsUserCheckable);
                    nameItem->setCheckState(Qt::Checked);
                }
                else{
                    nameItem->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled);
                    if(!newProfile && this->currentProfile.modExistsByFilename(mod.modFileName)){
                        nameItem->setCheckState(Qt::Checked);
                    }
                    else{
                        nameItem->setCheckState(Qt::Unchecked);
                    }

                }
                nameItem->setText(mod.modName);
                m_ui->campaignModList->setItem(m_ui->campaignModList->rowCount() -1,0,nameItem);
                fileItem = new QTableWidgetItem();
                fileItem->setText(mod.modFileName);
                m_ui->campaignModList->setItem(m_ui->campaignModList->rowCount() -1,1,fileItem);
                break;
            case Mod::Global:
                m_ui->globalModList->insertRow(m_ui->globalModList->rowCount());
                nameItem = new QTableWidgetItem();
                if(mod.autoload){
                    nameItem->setFlags(Qt::ItemIsUserCheckable);
                    nameItem->setCheckState(Qt::Checked);
                }
                else{
                    nameItem->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled);
                    if(!newProfile && this->currentProfile.modExistsByFilename(mod.modFileName)){
                        nameItem->setCheckState(Qt::Checked);
                    }
                    else{
                        nameItem->setCheckState(Qt::Unchecked);
                    }
                }
                nameItem->setText(mod.modName);
                m_ui->globalModList->setItem(m_ui->globalModList->rowCount() -1,0,nameItem);
                fileItem = new QTableWidgetItem();
                fileItem->setText(mod.modFileName);
                m_ui->globalModList->setItem(m_ui->globalModList->rowCount() -1,1,fileItem);
                break;
            case Mod::Multiplay:
                m_ui->multiplayModList->insertRow(m_ui->multiplayModList->rowCount());
                nameItem = new QTableWidgetItem();
                if(mod.autoload){
                    nameItem->setFlags(Qt::ItemIsUserCheckable);
                    nameItem->setCheckState(Qt::Checked);
                }
                else{
                    nameItem->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled);
                    if(!newProfile && this->currentProfile.modExistsByFilename(mod.modFileName)){
                        nameItem->setCheckState(Qt::Checked);
                    }
                    else{
                        nameItem->setCheckState(Qt::Unchecked);
                    }
                }
                nameItem->setText(mod.modName);
                m_ui->multiplayModList->setItem(m_ui->multiplayModList->rowCount() -1,0,nameItem);
                fileItem = new QTableWidgetItem();
                fileItem->setText(mod.modFileName);
                m_ui->multiplayModList->setItem(m_ui->multiplayModList->rowCount() -1,1,fileItem);
                break;
        }
    }
    delete gameManager;
}
void ModManager::on_cancelButton_clicked()
{
    this->reject();
}

void ModManager::on_okButton_clicked()
{
    if(validProfile()){
        this->m_ui->profileName->setText(this->m_ui->profileName->text().trimmed());
        saveProfile();
        this->accept();

    }
}
void ModManager::saveProfile(){
    ModProfile profile;
    profile.setProfileName(this->m_ui->profileName->text());
    for(int index = 0;index < m_ui->campaignModList->rowCount();index++){
        QTableWidgetItem *nameItem = m_ui->campaignModList->item(index,0);
        QTableWidgetItem *fileItem = m_ui->campaignModList->item(index,1);
        if(nameItem->checkState() == Qt::Checked && nameItem->flags().testFlag(Qt::ItemIsEnabled))
        {
            Mod mod;
            mod.modName = nameItem->text();
            mod.modType = Mod::Campaign;
            mod.autoload = false;
            mod.modFileName = fileItem->text();
            profile.addMod(mod);
        }

    }
    for(int index = 0;index < m_ui->multiplayModList->rowCount();index++){
        QTableWidgetItem *nameItem = m_ui->multiplayModList->item(index,0);
        QTableWidgetItem *fileItem = m_ui->multiplayModList->item(index,1);
        if(nameItem->checkState() == Qt::Checked && nameItem->flags().testFlag(Qt::ItemIsEnabled))
        {
            Mod mod;
            mod.modName = nameItem->text();
            mod.modType = Mod::Multiplay;
            mod.autoload = false;
            mod.modFileName = fileItem->text();
            profile.addMod(mod);
        }

    }
    for(int index = 0;index < m_ui->globalModList->rowCount();index++){
        QTableWidgetItem *nameItem = m_ui->globalModList->item(index,0);
        QTableWidgetItem *fileItem = m_ui->globalModList->item(index,1);
        if(nameItem->checkState() == Qt::Checked && nameItem->flags().testFlag(Qt::ItemIsEnabled))
        {
            Mod mod;
            mod.modName = nameItem->text();
            mod.modType = Mod::Global;
            mod.autoload = false;
            mod.modFileName = fileItem->text();
            profile.addMod(mod);
        }

    }
    if(this->newProfile){
        Settings::modProfiles.append(profile);
    }
    else{
        int profileIndex = Settings::modProfiles.indexOf(currentProfile,0);
        Settings::modProfiles.replace(profileIndex,profile);
    }
}
bool ModManager::validProfile(){
    bool validProfile = true;
    if(this->m_ui->profileName->text().trimmed().length() == 0){
        QMessageBox::warning(this,"Invalid Profile Name","Please enter a profile name.",QMessageBox::Ok,QMessageBox::NoButton);
        validProfile = false;
    }
    //check for unique name
    bool unique = true;
    for(int index = 0;index < Settings::modProfiles.size();index++){
        ModProfile profile = Settings::modProfiles.at(index);
        if(profile.getProfileName() == this->m_ui->profileName->text().trimmed() && profile.getProfileName() != this->currentProfile.getProfileName()){
            unique = false;
            index = Settings::modProfiles.size();
        }
    }
    if(!unique){
        QMessageBox::warning(this,"Invalid Profile Name","Profile name must be unique.",QMessageBox::Ok,QMessageBox::NoButton);
        validProfile = false;
    }

    return validProfile;
}

QString ModManager::getProfileName(){
    return this->m_ui->profileName->text();
}
