#include "modorderwindow.h"
#include "ui_modorderwindow.h"

ModOrderWindow::ModOrderWindow(QWidget *parent) :
    QDialog(parent),
    m_ui(new Ui::ModOrderWindow)
{
    m_ui->setupUi(this);
    m_ui->modOrderTree->setColumnHidden(2,true);
}

ModOrderWindow::~ModOrderWindow()
{
    delete m_ui;
}

void ModOrderWindow::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
void ModOrderWindow::on_cancelButton_clicked() {
    this->reject();
}

void ModOrderWindow::setProfile(ModProfile profile) {
    this->currentProfile = profile;
    for(int index = 0;index < this->currentProfile.orderedMods.size();index++) {
        Mod mod = this->currentProfile.orderedMods.at(index);
        QTreeWidgetItem *modItem = new QTreeWidgetItem(this->m_ui->modOrderTree);
        this->m_ui->modOrderTree->setColumnWidth(0,241);
        this->m_ui->modOrderTree->setColumnWidth(1,100);
        modItem->setText(0,mod.modName);
        modItem->setText(2,mod.modFileName);
        switch(mod.modType) {
            case Mod::Global:
                modItem->setText(1,"Global");
                break;
            case Mod::Multiplay:
                modItem->setText(1,"Multiplay");
                break;
            case Mod::Campaign:
                modItem->setText(1,"Campaign");
                break;

        }
    }
}

void ModOrderWindow::on_moveUpButton_clicked() {
    int currentIndex = this->m_ui->modOrderTree->currentIndex().row();
    if(currentIndex > 0) {
        QTreeWidgetItem *topItem = this->m_ui->modOrderTree->takeTopLevelItem(currentIndex);
        this->m_ui->modOrderTree->insertTopLevelItem(currentIndex -1,topItem);
        this->m_ui->modOrderTree->clearSelection();
        topItem->setSelected(true);
        this->m_ui->modOrderTree->setCurrentItem(topItem);
    }
}

void ModOrderWindow::on_moveDownButton_clicked() {
    int currentIndex = this->m_ui->modOrderTree->currentIndex().row();
    if(currentIndex < this->m_ui->modOrderTree->topLevelItemCount() - 1) {
        QTreeWidgetItem *topItem = this->m_ui->modOrderTree->takeTopLevelItem(currentIndex);
        this->m_ui->modOrderTree->insertTopLevelItem(currentIndex + 1,topItem);
        this->m_ui->modOrderTree->clearSelection();
        topItem->setSelected(true);
        this->m_ui->modOrderTree->setCurrentItem(topItem);
    }
}

void ModOrderWindow::on_okButton_clicked() {
    updateOrder();
    Settings *settings = new Settings();
    settings->updateProfile(this->currentProfile);
    this->accept();
}
void ModOrderWindow::updateOrder() {
    this->mods.clear();
    for(int index = 0;index < this->m_ui->modOrderTree->topLevelItemCount();index++) {
        QTreeWidgetItem *item = this->m_ui->modOrderTree->topLevelItem(index);
        Mod mod;
        mod.modFileName = item->text(2);
        mod.modName = item->text(0);
        if(item->text(1) == "Global") {
            mod.modType = Mod::Global;
        }
        else if(item->text(1) == "Multiplay") {
            mod.modType = Mod::Multiplay;
        }
        else if(item->text(1) == "Campaign") {
            mod.modType = Mod::Campaign;
        }
        this->mods.append(mod);
        this->currentProfile.orderedMods = this->mods;
    }
}

void ModOrderWindow::on_applyButton_clicked() {
    updateOrder();
}
