#include "settings.h"

QString Settings::warzonePath;
QString Settings::dataPath;
QString Settings::modPath;
QVector<class ModProfile> Settings::modProfiles;
QString Settings::selectedProfile;

bool Settings::dataDirEnabled;
QString Settings::dataDir;
bool Settings::configDirEnabled;
QString Settings::configDir;
bool Settings::gameTypeEnabled;
QString Settings::gameType;
bool Settings::savedGameEnabled;
QString Settings::savedGame;
bool Settings::cheatsEnabled;
bool Settings::hostScreen;
bool Settings::joinMPEnabled;
QString Settings::hostName;

bool Settings::fullscreen;
bool Settings::shadows;
bool Settings::resolutionEnabled;
int Settings::width;
int Settings::height;
bool Settings::sound;

bool Settings::logFileEnabled;
QString Settings::logFile;
bool Settings::debugAll;
bool Settings::debugMain;
bool Settings::debugSound;
bool Settings::debugWZ;
bool Settings::debug3D;
bool Settings::debugTexture;
bool Settings::debugMemory;
bool Settings::debugScript;
bool Settings::debugVideo;
bool Settings::debugNet;
bool Settings::debugError;
bool Settings::disableAsserts;
bool Settings::selfTest;
bool Settings::crash;
bool Settings::flushDebug;

Settings::Settings()
{
}

bool Settings::loadSettings(){
    bool settingsLoaded;

    QString settingsFile = "";
#ifdef Q_OS_LINUX
    settingsFile = QString(getenv("HOME")) + "/.wz2100launcher/settings.xml";
#endif

#ifdef Q_OS_MAC
    settingsFile = "";
#endif

#ifdef Q_OS_WIN
    settingsFile = QApplication::applicationDirPath() + "/settings.xml";
#endif
    QFile file(settingsFile);
    if (!file.exists() || !file.open(QFile::ReadOnly | QFile::Text)) {
        settingsLoaded = false;
    }
    else {
        QString errorStr;
        int errorLine;
        int errorColumn;

        QDomDocument doc;
        if(!doc.setContent(&file,false,&errorStr,&errorLine,&errorColumn)) {
            //std::cout << errorStr.toStdString();
        }

        QDomElement root = doc.firstChildElement();
        QDomElement child = root.firstChildElement();
        while(!child.isNull()) {
            if(child.nodeName() == "warzonePath") {
                Settings::warzonePath = child.firstChild().nodeValue();
            }
            else if(child.nodeName() == "modPath") {
                Settings::modPath = child.firstChild().nodeValue();
            }
            else if(child.nodeName() == "dataPath") {
                Settings::dataPath = child.firstChild().nodeValue();
            }

            else if(child.nodeName() == "selectedProfile") {
                Settings::selectedProfile = child.firstChild().nodeValue();
            }
            else if(child.nodeName() == "dataDirEnabled") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::dataDirEnabled = true;
                }
                else{
                    Settings::dataDirEnabled = false;
                }
            }
            else if(child.nodeName() == "dataDir") {
                Settings::dataDir = child.firstChild().nodeValue();
            }
            else if(child.nodeName() == "configDirEnabled") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::configDirEnabled = true;
                }
                else{
                    Settings::configDirEnabled = false;
                }
            }
            else if(child.nodeName() == "configDir") {
                Settings::configDir = child.firstChild().nodeValue();
            }
            else if(child.nodeName() == "gameTypeEnabled") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::gameTypeEnabled = true;
                }
                else{
                    Settings::gameTypeEnabled = false;
                }
            }
            else if(child.nodeName() == "gameType") {
                Settings::gameType = child.firstChild().nodeValue();
            }
            else if(child.nodeName() == "savedGameEnabled") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::savedGameEnabled = true;
                }
                else{
                    Settings::savedGameEnabled = false;
                }
            }
            else if(child.nodeName() == "savedGame") {
                Settings::savedGame = child.firstChild().nodeValue();
            }
            else if(child.nodeName() == "cheatsEnabled") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::cheatsEnabled = true;
                }
                else{
                    Settings::cheatsEnabled = false;
                }
            }
            else if(child.nodeName() == "hostScreen") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::hostScreen = true;
                }
                else{
                    Settings::hostScreen = false;
                }
            }
            else if(child.nodeName() == "joinMPEnabled") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::joinMPEnabled = true;
                }
                else{
                    Settings::joinMPEnabled = false;
                }
            }
            else if(child.nodeName() == "hostName") {
                Settings::hostName = child.firstChild().nodeValue();
            }
            else if(child.nodeName() == "logFile") {
                Settings::logFile = child.firstChild().nodeValue();
            }
            else if(child.nodeName() == "width") {
                Settings::width = child.firstChild().nodeValue().toInt();
            }
            else if(child.nodeName() == "height") {
                Settings::height = child.firstChild().nodeValue().toInt();
            }
            else if(child.nodeName() == "fullscreen") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::fullscreen = true;
                }
                else{
                    Settings::fullscreen = false;
                }
            }
            else if(child.nodeName() == "shadows") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::shadows = true;
                }
                else{
                    Settings::shadows = false;
                }
            }
            else if(child.nodeName() == "resolutionEnabled") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::resolutionEnabled = true;
                }
                else{
                    Settings::resolutionEnabled = false;
                }
            }
            else if(child.nodeName() == "sound") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::sound = true;
                }
                else{
                    Settings::sound = false;
                }
            }

            else if(child.nodeName() == "logFileEnabled") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::logFileEnabled = true;
                }
                else{
                    Settings::logFileEnabled = false;
                }
            }

            else if(child.nodeName() == "debugAll") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::debugAll = true;
                }
                else{
                    Settings::debugAll = false;
                }
            }

            else if(child.nodeName() == "debugMain") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::debugMain = true;
                }
                else{
                    Settings::debugMain = false;
                }
            }
            else if(child.nodeName() == "debugSound") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::debugSound = true;
                }
                else{
                    Settings::debugSound = false;
                }
            }
            else if(child.nodeName() == "debugWZ") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::debugWZ = true;
                }
                else{
                    Settings::debugWZ = false;
                }
            }
            else if(child.nodeName() == "debug3D") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::debug3D = true;
                }
                else{
                    Settings::debug3D = false;
                }
            }
            else if(child.nodeName() == "debugTexture") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::debugTexture = true;
                }
                else{
                    Settings::debugTexture = false;
                }
            }
            else if(child.nodeName() == "debugMemory") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::debugMemory = true;
                }
                else{
                    Settings::debugMemory = false;
                }
            }
            else if(child.nodeName() == "debugScript") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::debugScript = true;
                }
                else{
                    Settings::debugScript = false;
                }
            }
            else if(child.nodeName() == "debugVideo") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::debugVideo = true;
                }
                else{
                    Settings::debugVideo = false;
                }
            }
            else if(child.nodeName() == "debugNet") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::debugNet = true;
                }
                else{
                    Settings::debugNet = false;
                }
            }
            else if(child.nodeName() == "debugError") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::debugError = true;
                }
                else{
                    Settings::debugError = false;
                }
            }
            else if(child.nodeName() == "disableAsserts") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::disableAsserts = true;
                }
                else{
                    Settings::disableAsserts = false;
                }
            }
            else if(child.nodeName() == "selfTest") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::selfTest = true;
                }
                else{
                    Settings::selfTest = false;
                }
            }
            else if(child.nodeName() == "crash") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::crash = true;
                }
                else{
                    Settings::crash = false;
                }
            }
            else if(child.nodeName() == "flushDebug") {
                if(child.firstChild().nodeValue() == "true"){
                    Settings::flushDebug = true;
                }
                else{
                    Settings::flushDebug = false;
                }
            }
            else if(child.nodeName() == "profiles"){
                QDomElement profileChild = child.firstChildElement();
                while(!profileChild.isNull()){
                    ModProfile profile;
                    profile.setProfileName(profileChild.firstChild().firstChild().nodeValue());
                    QDomElement modEl = profileChild.firstChild().nextSiblingElement().firstChildElement();
                    while(!modEl.isNull()){
                        Mod mod;
                        mod.modName = modEl.firstChild().firstChild().nodeValue();
                        mod.modFileName = modEl.firstChild().nextSibling().firstChild().nodeValue();
                        switch(modEl.firstChild().nextSiblingElement().nextSiblingElement().firstChild().nodeValue().toInt()){
                            case 1:
                                mod.modType = Mod::Global;
                                break;
                            case 2:
                                mod.modType = Mod::Multiplay;
                                break;
                            case 3:
                                mod.modType = Mod::Campaign;
                                break;
                        }

                        if(modEl.firstChild().nextSiblingElement().nextSiblingElement().nextSiblingElement().firstChild().nodeValue() == "true"){
                            mod.autoload = true;
                        }
                        else{
                            mod.autoload = false;
                        }
                        profile.addMod(mod);
                        modEl = modEl.nextSiblingElement();
                    }

                    modEl = profileChild.firstChild().nextSiblingElement().nextSiblingElement().firstChildElement();
                    profile.orderedMods.clear();
                    while(!modEl.isNull()){
                        Mod mod;
                        mod.modName = modEl.firstChild().firstChild().nodeValue();
                        mod.modFileName = modEl.firstChild().nextSibling().firstChild().nodeValue();
                        switch(modEl.firstChild().nextSiblingElement().nextSiblingElement().firstChild().nodeValue().toInt()){
                            case 1:
                                mod.modType = Mod::Global;
                                break;
                            case 2:
                                mod.modType = Mod::Multiplay;
                                break;
                            case 3:
                                mod.modType = Mod::Campaign;
                                break;
                        }
                        if(modEl.firstChild().nextSiblingElement().nextSiblingElement().nextSiblingElement().firstChild().nodeValue() == "true"){
                            mod.autoload = true;
                        }
                        else{
                            mod.autoload = false;
                        }
                        profile.orderedMods.append(mod);
                        modEl = modEl.nextSiblingElement();
                    }

                    modProfiles.append(profile);
                    profileChild = profileChild.nextSiblingElement();
                }

            }

            child = child.nextSiblingElement();
        }
        file.close();
        settingsLoaded = true;
    }




    //override saved settings
    if(!settingsLoaded && dataPath.length() == 0){
         GameManager *gm = new GameManager();
         dataPath = gm->findDataPath();
         delete gm;
    }
    QFile *optionsFile = new QFile(dataPath + "/config");
    optionsFile->open(QFile::ReadOnly);
    QTextStream stream(optionsFile);

    while(!stream.atEnd()){
        QString line = stream.readLine(0);
        if(line.indexOf("width") > -1){
            width = line.mid(line.indexOf("=") + 1,line.length() - line.indexOf("=")).toInt();
        }
        else if(line.indexOf("height") > -1){
            height = line.mid(line.indexOf("=") + 1,line.length() - line.indexOf("=")).toInt();
        }
        else if(line.indexOf("fullscreen") > -1){
            QString value = line.mid(line.indexOf("=") + 1,line.length() - line.indexOf("="));
            if(value == "1"){
                fullscreen = true;
            }
            else{
                fullscreen = false;
            }
        }
        else if(line.indexOf("sound") > -1){
            QString value = line.mid(line.indexOf("=") + 1,line.length() - line.indexOf("="));
            if(value == "1"){
                sound = true;
            }
            else{
                sound = false;
            }
        }
        else if(line.indexOf("shadows") > -1){
            QString value = line.mid(line.indexOf("=") + 1,line.length() - line.indexOf("="));
            if(value == "1"){
                shadows = true;
            }
            else{
                shadows = false;
            }
        }
    }
    optionsFile->close();

    //get WZ version
    //Warzone 2100 - Version 2.3_beta9

    return settingsLoaded;
}
bool Settings::saveSettings(){
    bool settingsSaved;


    const int indentSize = 4;

    QString settingsFile = "";
#ifdef Q_OS_LINUX
    settingsFile = QString(getenv("HOME")) + "/.wz2100launcher/settings.xml";
#endif

#ifdef Q_OS_MAC
    settingsFile = "";
#endif

#ifdef Q_OS_WIN
    settingsFile = QApplication::applicationDirPath() + "/settings.xml";
#endif
    QFile file(settingsFile);

    if (!file.open(QFile::WriteOnly | QFile::Text)) {
        settingsSaved = false;
    }
    else {
        QDomDocument doc;

        QDomElement root = doc.createElement("settings");
        doc.appendChild(root);


        QDomElement element = doc.createElement("warzonePath");
        QDomText elementT = doc.createTextNode(Settings::warzonePath);
        element.appendChild(elementT);
        root.appendChild(element);


        element = doc.createElement("dataPath");
        elementT = doc.createTextNode(Settings::dataPath);
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("modPath");
        elementT = doc.createTextNode(Settings::modPath);
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("selectedProfile");
        elementT = doc.createTextNode(Settings::selectedProfile);
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("dataDirEnabled");
        elementT = doc.createTextNode(QString(Settings::dataDirEnabled?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("dataDir");
        elementT = doc.createTextNode(Settings::dataDir);
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("configDirEnabled");
        elementT = doc.createTextNode(QString(Settings::configDirEnabled?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("configDir");
        elementT = doc.createTextNode(Settings::configDir);
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("gameTypeEnabled");
        elementT = doc.createTextNode(QString(Settings::gameTypeEnabled?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("gameType");
        elementT = doc.createTextNode(Settings::gameType);
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("savedGameEnabled");
        elementT = doc.createTextNode(QString(Settings::savedGameEnabled?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("savedGame");
        elementT = doc.createTextNode(Settings::savedGame);
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("cheatsEnabled");
        elementT = doc.createTextNode(QString(Settings::cheatsEnabled?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("hostScreen");
        elementT = doc.createTextNode(QString(Settings::hostScreen?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("joinMPEnabled");
        elementT = doc.createTextNode(QString(Settings::joinMPEnabled?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("hostName");
        elementT = doc.createTextNode(Settings::hostName);
        element.appendChild(elementT);
        root.appendChild(element);


        element = doc.createElement("fullscreen");
        elementT = doc.createTextNode(QString(Settings::fullscreen?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("shadows");
        elementT = doc.createTextNode(QString(Settings::shadows?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("resolutionEnabled");
        elementT = doc.createTextNode(QString(Settings::resolutionEnabled?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("width");
        QString val;
        val.setNum(Settings::width);
        elementT = doc.createTextNode(val);
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("height");
        val.setNum(Settings::height);
        elementT = doc.createTextNode(val);
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("sound");
        elementT = doc.createTextNode(QString(Settings::sound?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("logFileEnabled");
        elementT = doc.createTextNode(QString(Settings::logFileEnabled?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("logFile");
        elementT = doc.createTextNode(Settings::logFile);
        element.appendChild(elementT);
        root.appendChild(element);


        element = doc.createElement("debugAll");
        elementT = doc.createTextNode(QString(Settings::debugAll?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("debugMain");
        elementT = doc.createTextNode(QString(Settings::debugMain?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("debugSound");
        elementT = doc.createTextNode(QString(Settings::debugSound?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("debugWZ");
        elementT = doc.createTextNode(QString(Settings::debugWZ?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("debug3D");
        elementT = doc.createTextNode(QString(Settings::debug3D?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);


        element = doc.createElement("debugTexture");
        elementT = doc.createTextNode(QString(Settings::debugTexture?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("debugMemory");
        elementT = doc.createTextNode(QString(Settings::debugMemory?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("debugScript");
        elementT = doc.createTextNode(QString(Settings::debugScript?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("debugVideo");
        elementT = doc.createTextNode(QString(Settings::debugVideo?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("debugNet");
        elementT = doc.createTextNode(QString(Settings::debugNet?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("debugError");
        elementT = doc.createTextNode(QString(Settings::debugError?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("disableAsserts");
        elementT = doc.createTextNode(QString(Settings::disableAsserts?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("selfTest");
        elementT = doc.createTextNode(QString(Settings::selfTest?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("crash");
        elementT = doc.createTextNode(QString(Settings::crash?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("flushDebug");
        elementT = doc.createTextNode(QString(Settings::flushDebug?"true":"false"));
        element.appendChild(elementT);
        root.appendChild(element);

        element = doc.createElement("profiles");

        for(int index = 0;index < modProfiles.size();index++){
            ModProfile profile = modProfiles.at(index);
            QDomElement profileElement = doc.createElement("profile");
            QDomElement nameElement = doc.createElement("profileName");
            QDomText nameText = doc.createTextNode(profile.getProfileName());
            nameElement.appendChild(nameText);
            profileElement.appendChild(nameElement);

            QDomElement modsEl =  doc.createElement("mods");
            for(int index2 = 0;index2 < profile.selectedMods.size();index2++){
                Mod mod = profile.selectedMods.at(index2);

                QDomElement modEl = doc.createElement("mod");

                QDomElement modNameEl = doc.createElement("modName");
                QDomText modNameText = doc.createTextNode(mod.modName);
                modNameEl.appendChild(modNameText);
                modEl.appendChild(modNameEl);

                QDomElement modFileNameEl = doc.createElement("modFileName");
                QDomText modFileNameText = doc.createTextNode(mod.modFileName);
                modFileNameEl.appendChild(modFileNameText);
                modEl.appendChild(modFileNameEl);

                QDomElement modTypeEl = doc.createElement("modType");
                QString val;
                val.setNum(mod.modType);
                QDomText modTypeText = doc.createTextNode(val);
                modTypeEl.appendChild(modTypeText);
                modEl.appendChild(modTypeEl);

                QDomElement modAutoEl = doc.createElement("autoload");
                QDomText modAutoText = doc.createTextNode(QString(mod.autoload?"true":"false"));
                modAutoEl.appendChild(modAutoText);
                modEl.appendChild(modAutoEl);

                modsEl.appendChild(modEl);
            }
            profileElement.appendChild(modsEl);

            QDomElement modOrderEl =  doc.createElement("modorder");
            for(int index2 = 0;index2 < profile.orderedMods.size();index2++){
                Mod mod = profile.orderedMods.at(index2);

                QDomElement modEl = doc.createElement("mod");

                QDomElement modNameEl = doc.createElement("modName");
                QDomText modNameText = doc.createTextNode(mod.modName);
                modNameEl.appendChild(modNameText);
                modEl.appendChild(modNameEl);

                QDomElement modFileNameEl = doc.createElement("modFileName");
                QDomText modFileNameText = doc.createTextNode(mod.modFileName);
                modFileNameEl.appendChild(modFileNameText);
                modEl.appendChild(modFileNameEl);

                QDomElement modTypeEl = doc.createElement("modType");
                QString val;
                val.setNum(mod.modType);
                QDomText modTypeText = doc.createTextNode(val);
                modTypeEl.appendChild(modTypeText);
                modEl.appendChild(modTypeEl);

                QDomElement modAutoEl = doc.createElement("autoload");
                QDomText modAutoText = doc.createTextNode(QString(mod.autoload?"true":"false"));
                modAutoEl.appendChild(modAutoText);
                modEl.appendChild(modAutoEl);

                modOrderEl.appendChild(modEl);
            }
            profileElement.appendChild(modOrderEl);
            element.appendChild(profileElement);

        }
        root.appendChild(element);

        QTextStream out(&file);

        doc.save(out, indentSize);
        file.close();
        settingsSaved = true;
    }



    return settingsSaved;
}

void Settings::removeProfile(QString profileName){
    for(int index = 0;index < modProfiles.size();index++){
        ModProfile profile = modProfiles.at(index);
        if(profile.getProfileName() == profileName){
            modProfiles.remove(index);
            index = modProfiles.size();
        }
    }
}
ModProfile Settings::getProfile(QString profileName){
    ModProfile profile;
    for(int index = 0;index < modProfiles.size();index++){
        profile = modProfiles.at(index);
        if(profile.getProfileName() == profileName){
            index = modProfiles.size();
        }
    }
    return profile;
}
void Settings::updateProfile(ModProfile profile){
    for(int index = 0;index < modProfiles.size();index++){
        ModProfile profile2 = modProfiles.at(index);
        if(profile2.getProfileName() == profile.getProfileName()){
            modProfiles.replace(index,profile);
            index = modProfiles.size();
        }
    }
}
